from red_green_refactor_widget.app import App
from red_green_refactor_widget.app import GreenStage
from red_green_refactor_widget.app import RedStage
from red_green_refactor_widget.app import RefactorStage
from red_green_refactor_widget.ui import UI
import PyQt5
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QKeyEvent
import unittest
from unittest import mock

class TestApp(unittest.TestCase):

    def setUp(self):
        self.ui_class_patcher = mock.patch(
            "red_green_refactor_widget.app.UI", autospec=True)
        self.ui_class_patcher.start()
        self.app = App()

    def tearDown(self):
        self.ui_class_patcher.stop()

    def test_initial_stage_is_red(self):
        self.assertIsInstance(self.app.stage, RedStage)

    def test_advance_from_red_to_green(self):
        self.app.advance_stage()
        self.assertIsInstance(self.app.stage, GreenStage)

    def test_advance_from_green_to_refactor(self):
        self.app.advance_stage()
        self.app.advance_stage()
        self.assertIsInstance(self.app.stage, RefactorStage)

    def test_advance_from_refactor_to_red(self):
        self.app.advance_stage()
        self.app.advance_stage()
        self.app.advance_stage()
        self.assertIsInstance(self.app.stage, RedStage)

    def test_app_has_ui(self):
        self.assertIsInstance(self.app.ui, UI)

    def test_run_ui(self):
        self.app.run()
        self.app.ui.run.assert_called_once();

    def test_apply_stage_to_ui(self):
        self.app.apply_stage_class(RedStage)
        ui_apply_stage_first_argument = self.app.ui.apply_stage.call_args[0][0]
        self.assertIsInstance(ui_apply_stage_first_argument, RedStage)

    def test_set_advance_stage_callback(self):
        self.app.ui.set_advance_stage_callback.assert_called_once()

class TestUI(unittest.TestCase):

    def setUp(self):
        self.qapplication_patcher = mock.patch(
            "red_green_refactor_widget.ui.QApplication", autospec=True)
        self.qapplication_patcher.start()
        self.uic_patcher = mock.patch(
            "red_green_refactor_widget.ui.uic", autospec=True)
        self.uic_mock = self.uic_patcher.start()
        self.uic_mock.loadUi.return_value = mock.create_autospec(
            PyQt5.QtWidgets.QMainWindow)
        self.register_hotkey_patcher = mock.patch(
            "red_green_refactor_widget.ui.UI.register_hotkey")
        self.register_hotkey_patcher.start()
        self.ui = UI()

    def tearDown(self):
        self.register_hotkey_patcher.stop()
        self.uic_patcher.stop()
        self.qapplication_patcher.stop()

    def test_has_qapplication(self):
        self.assertIsInstance(self.ui.app, PyQt5.QtWidgets.QApplication)

    def test_has_window(self):
        self.assertIsInstance(self.ui.window, PyQt5.QtWidgets.QMainWindow)

    def test_run_app_exec(self):
        self.ui.run()
        self.ui.app.exec_.assert_called_once()

    def test_create_tool_window(self):
        self.ui.window.setWindowFlag.assert_any_call(Qt.Tool)
        self.ui.window.setAttribute.assert_called_with(Qt.WA_QuitOnClose)

    @mock.patch("red_green_refactor_widget.ui.UI.set_stage_name")
    @mock.patch("red_green_refactor_widget.ui.UI.set_stage_hint")
    @mock.patch("red_green_refactor_widget.ui.UI.set_stage_color")
    def test_apply_red_stage(
            self,
            mock_set_stage_color,
            mock_set_stage_hint,
            mock_set_stage_name):
        stage = RedStage()
        self.ui.apply_stage(stage)
        self.ui.set_stage_name.assert_called_with(stage.name)
        self.ui.set_stage_hint.assert_called_with(stage.hint)
        self.ui.set_stage_color.assert_called_with(stage.color)

    @mock.patch("red_green_refactor_widget.ui.UI.set_stage_name")
    @mock.patch("red_green_refactor_widget.ui.UI.set_stage_hint")
    @mock.patch("red_green_refactor_widget.ui.UI.set_stage_color")
    def test_apply_stages(
            self,
            mock_set_stage_color,
            mock_set_stage_hint,
            mock_set_stage_name):
        stages = [ RedStage, GreenStage, RefactorStage ]
        for stage_class in stages:
            stage = stage_class()
            self.ui.apply_stage(stage)
            self.ui.set_stage_name.assert_called_with(stage.name)
            self.ui.set_stage_hint.assert_called_with(stage.hint)
            self.ui.set_stage_color.assert_called_with(stage.color)

    def test_set_stage_name(self):
        self.ui.set_stage_name("Red")
        self.ui.stage_name_label.setText.assert_called_with("Red")

    def test_set_stage_hint(self):
        self.ui.set_stage_hint("Some hint")
        self.ui.stage_hint_label.setText.assert_called_with("Some hint")

    def test_advance_stage_keyboard_shortcut(self):
        callback = mock.Mock()
        self.ui.set_advance_stage_callback(callback)
        self.ui.on_advance_stage_keyboard_event()
        callback.assert_called_once()

    def test_register_hotkey(self):
        self.ui.register_hotkey.assert_called_once()



from . import debug
import os
from pyqtkeybind import keybinder
from PyQt5 import uic
from PyQt5.QtCore import QAbstractEventDispatcher
from PyQt5.QtCore import QAbstractNativeEventFilter
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QApplication
from PyQt5.QtWidgets import QLabel
from PyQt5.QtWidgets import QMainWindow
import sys

class UI:

    def __init__(self):
        self.app = QApplication(sys.argv)
        ui_dir = os.path.dirname(os.path.abspath(__file__))
        ui_path = os.path.join(ui_dir, "window.ui")
        self.window = uic.loadUi(ui_path)
        assert isinstance(self.window, QMainWindow)
        self.window.setWindowFlag(Qt.Tool)
        self.window.setWindowFlag(Qt.WindowStaysOnTopHint)
        self.window.setAttribute(Qt.WA_QuitOnClose)
        self.stage_name_label = self.window.findChild(QLabel, "stage_name")
        assert self.stage_name_label
        self.stage_hint_label = self.window.findChild(QLabel, "stage_hint")
        assert self.stage_hint_label
        self.register_hotkey()
        self.window.show()

    def register_hotkey(self):
        keybinder.init()
        keybinder.register_hotkey(
            self.window.winId(),
            "Alt+F12",
            self.on_advance_stage_keyboard_event)
        self.event_filter = NativeEventFilter(keybinder)
        self.event_dispatcher = QAbstractEventDispatcher.instance()
        self.event_dispatcher.installNativeEventFilter(self.event_filter)

    def run(self):
        self.app.exec_()

    def apply_stage(self, stage):
        self.set_stage_name(stage.name)
        self.set_stage_hint(stage.hint)
        self.set_stage_color(stage.color)

    def set_stage_name(self, name):
        self.stage_name_label.setText(name)

    def set_stage_hint(self, hint):
        self.stage_hint_label.setText(hint)

    def set_stage_color(self, color):
        self.window.setStyleSheet("background-color: {}".format(color))

    def set_advance_stage_callback(self, callback):
        self.advance_stage_callback = callback

    def on_advance_stage_keyboard_event(self):
        self.advance_stage_callback()

class NativeEventFilter(QAbstractNativeEventFilter):

    def __init__(self, keybinder):
        self.keybinder = keybinder
        super().__init__()

    def nativeEventFilter(self, eventType, message):
        ret = self.keybinder.handler(eventType, message)
        return ret, 0



import inspect

def not_implemented():
    prev_frame = inspect.currentframe().f_back
    prev_frame_info = inspect.getframeinfo(prev_frame)
    print("{}:{} Not implemented"
        .format(prev_frame_info.filename, prev_frame_info.lineno))



from .ui import UI

class App:

    def __init__(self):
        self.ui = UI()
        self.ui.set_advance_stage_callback(self.advance_stage)
        self.apply_stage_class(RedStage)

    def run(self):
        self.ui.run()

    def apply_stage_class(self, stage_class):
        self.stage = stage_class()
        self.ui.apply_stage(self.stage)

    def advance_stage(self):
        if isinstance(self.stage, RedStage):
            self.apply_stage_class(GreenStage)
        elif isinstance(self.stage, GreenStage):
            self.apply_stage_class(RefactorStage)
        elif isinstance(self.stage, RefactorStage):
            self.apply_stage_class(RedStage)

class RedStage:

    def __init__(self):
        self.name = "RED"
        self.hint = "Write a test, watch it fail"
        self.color = "red"

class GreenStage:

    def __init__(self):
        self.name = "GREEN"
        self.hint = "Write just enough code to pass the test"
        self.color = "green"

class RefactorStage:

    def __init__(self):
        self.name = "REFACTOR"
        self.hint = "Improve the code without changing its behavior"
        self.color = "blue"



import setuptools

def get_long_description():
    with open("README.md", "r") as readme_file:
        return readme_file.read()

setuptools.setup(
    name="red-green-refactor-widget",
    version="0.0.1",
    author="Andrey Krainyak",
    author_email="mode.andrew@gmail.com",
    description=\
        "A simple widget that will help you to stick to "
        "the red-green-refactor workflow",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    url="http://github.com/mode89/red-green-refactor-widget",
    packages=setuptools.find_packages(),
    package_data={ "red_green_refactor_widget": [ "window.ui" ] },
    include_pacakge_data=True,
    install_requires=[
        "PyQt5",
        "pyqtkeybind",
    ],
    entry_points={
        "console_scripts":
            ["red-green-refactor-widget=red_green_refactor_widget.cli:main"],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6"
)
